/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "iar_bus.h"
#define USE_IAR_BUS
#  include "camera.h"
#include "camera_ib_pk.h"
#include "my_stdio.h"

static int time_out = 10;

static int my_ibus_read(int fd, struct ib_packet *d);

static int ibus_dummy_read(int fd)
{
	int retval;
	struct timeval tv;
	fd_set rfds;
        int fd_max = fd + 1;
	struct ib_packet d;

	do {
		tv.tv_sec = 0;
		tv.tv_usec = 10;
                FD_ZERO(&rfds);
                FD_SET(fd, &rfds);
		retval = select(fd_max, &rfds, NULL, NULL, &tv);
		if (retval) {
                        if( FD_ISSET(fd, &rfds) ) {
				my_ibus_read(fd, &d);
                        }
		}
	} while(retval);

	return 0;
}


static int ciic_send_data( int dev, void *data, int len)
{
	int ret;
	struct ib_packet d;

	ibus_dummy_read(dev);
	do {
		if ( (ret = ibus_send_data_to(dev, CC_IB_ID, data, len)) < 0){
			return -1;
		}
	} while( my_ibus_read(dev, &d) < 0);
	
	return ret;
//	return ibus_send_data_to(dev, CC_IB_ID, data, len);
}




static ssize_t soft_read_time_out(struct timeval tv, int fd, void *buf,
                                  size_t count)
{
        fd_set rfds;
        int fd_max = fd + 1;
        int retval, l = 0;
        struct timeval tvl;

        while ( l < count) {
		memcpy(&tvl, &tv, sizeof(struct timeval));
                FD_ZERO(&rfds);
                FD_SET(fd, &rfds);
		//printf("Time: %02ld.%06ld - ", tvl.tv_sec, tvl.tv_usec);
                retval = select(fd_max, &rfds, NULL, NULL, &tvl);
                if (retval) {
                        if( FD_ISSET(fd, &rfds) ) {
				retval = read(fd, buf+l, count - l);
				l += retval;
                        }
                } else { /* FIXME: .... */
			PVERB(PERR, "read time out.\n");
			//printf("read time out.\n");
			//return -1;
		}
		//printf("%02ld.%06ld\n", tvl.tv_sec, tvl.tv_usec);
        }

#ifdef DEBUG_MODE
        if (verbose_flag & PINFO && l < 42) {
                int __i;
                PVERB(PINFO, "# camera_io_ib_client.c read <- ");
                for(__i=0;__i<l;__i++) {
			if (__i == 16) PVERB(PINFO, "\n# ");
			PVERB(PINFO, "%02x ",
			      *(((u_char *)buf)+__i) );
		}
                PVERB(PINFO, "\n");
        }
#endif
        return l;
}



ssize_t soft_read(int fd, void *buf, size_t count)
{
        struct timeval tv;

        tv.tv_sec = time_out;
        tv.tv_usec = 0;

        return soft_read_time_out(tv, fd, buf, count);
}


static int my_ibus_read(int fd, struct ib_packet *d)
{
	int ret;
	int result;
	struct cc_packet *dp;

	memset(d, 0, sizeof(struct ib_packet));
	dp = (struct cc_packet *) d->data;
        ret = result = soft_read(fd, d, sizeof(struct ib_packet_head) );
        if( d->h.size && result > 0){
                result = soft_read(fd, d->data, d->h.size );
		if (result > 0) {
			ret += result;
		} else {
			ret = -1;
		}
        }
	if (ret > 0) {
		if( dp->h.cmd == CC_OK){
			return 0;
		}else if (dp->h.cmd == CC_SLEEP) {
			PVERB(PINFO,"my_ibus_read: sleeping %d sec.\n",
			      atoi(dp->data));
			sleep(atoi(dp->data));
			return -1;//my_ibus_read(fd, d);
		}
	}

        return ret;
}


static int my_ibus_get_ret(int fd)
{
	struct ib_packet d;
	struct cc_packet *dp;
	int ret;

	dp = (struct cc_packet *) d.data;
	do {
		ret = my_ibus_read(fd, &d);
		if (dp->h.cmd != CC_RETURN && dp->h.cmd != 0)
			PVERB(PERR,
			      "my_ibus_get_ret: packet out of sequence (%x)\n",
			      dp->h.cmd );
		/* FIXME: ret */
	} while (dp->h.cmd != CC_RETURN && ret >= 0 /*|| ret < 0*/);

        return atoi(dp->data);
}



int get_int_from_cc(int dev, char *cmd, int *val)
{
	struct ib_packet pk;
	struct cc_packet *ccp;
	int ret;
	int l;

	memset(&pk, 0, sizeof(struct ib_packet));
	ccp = (struct cc_packet *)&pk.data;
	l = sizeof(struct cc_packet_head);
	l += sprintf(ccp->data,"%s", cmd);
	ccp->h.cmd = CC_GET_INT;
	ccp->h.size = 0;
	ciic_send_data(dev, ccp, l);

	ret = my_ibus_read(dev, &pk);
	if( ret > 0) {
		*val = atoi(ccp->data);
		PVERB(PINFO,"# val get: %d\n", *val);
	}

	return my_ibus_get_ret(dev);		
}


int get_str_from_cc(int dev, char *cmd, char *str, int len)
{
	struct ib_packet pk;
	struct cc_packet *ccp;
	int ret;
	int l;
	
	memset(&pk, 0, sizeof(struct ib_packet));
	ccp = (struct cc_packet *)&pk.data;
	l = sizeof(struct cc_packet_head);
	l += sprintf(ccp->data,"%s", cmd);
	ccp->h.cmd = CC_GET_STR;
	ccp->h.size = len & PACKET_HEAD_SIZE_MASK;
	ciic_send_data(dev, ccp, l);

	ret = my_ibus_read(dev, &pk);
	if( ret > 0) {
		memcpy(str, ccp->data, len);
	}

	return my_ibus_get_ret(dev);

}


int put_int_to_cc(int dev, char *cmd, int val)
{
	struct cc_packet cc;
	int l;

	memset(&cc, 0, sizeof(struct cc_packet));
	l = sizeof(struct cc_packet_head);
	l += sprintf(cc.data,"%c%d%c", cmd[0], val, 0);
	cc.h.cmd = CC_PUT_INT;
	cc.h.size = 0;

	if (ciic_send_data(dev, &cc, l) < 0){
		return -1;
	}

	return my_ibus_get_ret(dev);
}


int put_str_to_cc(int dev, char *cmd, char *str)
{
	struct cc_packet cc;
	int l;

	memset(&cc, 0, sizeof(struct cc_packet));
	l = sizeof(struct cc_packet_head);
	l += sprintf(cc.data,"%c%s", cmd[0], str);
	cc.h.cmd = CC_PUT_STR;
	cc.h.size = 0;
	
	if (ciic_send_data(dev, &cc, l) < 0) {
		return -1;
	}

	return my_ibus_get_ret(dev);
}


int action_to_cc(int dev, char *cmd)
{
	struct cc_packet ccp;
	int l;

	memset(&ccp, 0, sizeof(struct cc_packet));
	l = sizeof(struct cc_packet_head);
	l++;
	ccp.data[0] =  cmd[0];
	ccp.data[1] = 0;
	ccp.h.cmd = CC_ACTION;
	ccp.h.size = 0;
	if ( ciic_send_data(dev, &ccp, l) < 0) {
		printf("action_to_cc: FAIL\n");
		return -1;
	}

	return my_ibus_get_ret(dev);
}


int send_data_to_cc(int dev, char *data, int len)
{
	struct cc_packet ccp;
	int l;
	int i;

	memset(&ccp, 0, sizeof(struct cc_packet));
	l = sizeof(struct cc_packet_head);
	l += sprintf(ccp.data,"%d%c", len, 0);
	ccp.h.cmd = CC_SEND_DATA;
	ccp.h.size = 0;
	if ( ciic_send_data(dev, &ccp, l) < 0) {
		return -1;
	}
	for(i=0; i < len; i += CC_PACKET_DATA_LEN) {
		if( (len - i - CC_PACKET_DATA_LEN) > 0){
			l = CC_PACKET_DATA_LEN;
		}else{
			l = len - i;
		}
		memcpy(ccp.data, data+i, l);
		ccp.h.size = l;
		l += sizeof(struct cc_packet_head);	
		usleep(100000); /* FIXME: why ? */
#ifdef SEND_DATA_ATOMIC
		if ( ibus_send_data_to(dev, CC_IB_ID, &ccp, l) < 0){
			return -1;
		}
#else
		if ( ciic_send_data(dev, &ccp, l) < 0) {
			printf("action_to_cc: FAIL\n");
			return -1;
		}
#endif
	}

	return my_ibus_get_ret(dev);
}


int rcv_data_from_cc(int dev, char *data, int *len)
{
	struct ib_packet pk;
	struct cc_packet *ccp;
	int ret, rlen;
	int l;
	struct timeval tv;
	fd_set rfds;
        int fd_max = dev + 1;


	memset(&pk, 0, sizeof(struct ib_packet));
	ccp = (struct cc_packet *)&pk.data;
	l = sizeof(struct cc_packet_head);
	ccp->h.cmd = CC_RCV_DATA;
	ccp->h.size = *len & PACKET_HEAD_SIZE_MASK;
	if ( ccp->h.size != *len ) {
		PVERB(PERR, "rcv_data_from_cc: size exeded, %d > %d\n",
		      *len, PACKET_HEAD_SIZE_MASK);
		return -1;
	}
	ciic_send_data(dev, ccp, l);
	
	/* Wait for data arrival*/
	rlen = -1;
	tv.tv_sec = 6; /* <- FIXME */
        tv.tv_usec = 0;
	FD_ZERO(&rfds);
	FD_SET(dev, &rfds);
	ret = select(fd_max, &rfds, NULL, NULL, &tv);
	if (ret) {
		rlen = 0;
		while (rlen < *len) {
			ret = my_ibus_read(dev, &pk);
			if( ret < 0 ) {
				PVERB(PERR,"rcv_data_client: ERROR %d\n", ret);
				//time_out = old_time_out;
				return ret;
			}
			ret -=  sizeof(struct ib_packet_head) + 
				sizeof(struct cc_packet_head);
			PVERB(PINFO,"rcv_data_client: %d\n", ret);
			if( ret > 0) {
				memcpy(data+rlen, ccp->data, ret);
				rlen += ret;
			}else{
				break;
			}
		}
		*len = rlen;
		ret = my_ibus_get_ret(dev);
		if( rlen != ret )
			PVERB(PERR,
			      "rcv_data_from_cc: Bad size: read: %d. expected %d.\n",
			      rlen, ret);
	
	}

	return rlen;
}
	

int reset_cc(int dev)
{
	struct cc_packet ccp;
	int l;
	
	memset(&ccp, 0, sizeof(struct cc_packet));
	l = sizeof(struct cc_packet_head);
	ccp.h.cmd = CC_ABORT;
	ccp.h.size = 0;
	if ( ciic_send_data(dev, &ccp, l) < 0) {
		printf("action_to_cc: FAIL\n");
		return -1;
	}

	return my_ibus_get_ret(dev);

}


int rcv_setup_from_cc(int dev, struct camera_setup *set)
{
	struct ib_packet pk;
	struct cc_packet *ccp;
	int ret;
	int l;

	memset(&pk, 0, sizeof(struct ib_packet));
	ccp = (struct cc_packet *)&pk.data;
	l = sizeof(struct cc_packet_head);
	ccp->h.cmd = CC_RCV_SETUP;
	ccp->h.size = sizeof(struct camera_setup);
	ciic_send_data(dev, ccp, l);

	ret = my_ibus_read(dev, &pk);
	if( ret > 0) {
		memcpy(set, ccp->data, sizeof(struct camera_setup));
		ret = my_ibus_get_ret(dev);
	}

	return ret;
}


int rcv_mouse_from_cc(int dev, struct mouse_pointer *mp)
{
	struct ib_packet pk;
	struct cc_packet *ccp;
	int ret;
	int l;

	memset(&pk, 0, sizeof(struct ib_packet));
	ccp = (struct cc_packet *)&pk.data;
	l = sizeof(struct cc_packet_head);
	ccp->h.cmd = CC_RCV_MOUSE;
	ccp->h.size = sizeof(struct mouse_pointer);
	ciic_send_data(dev, ccp, l);

	ret = my_ibus_read(dev, &pk);
	if( ret > 0) {
		memcpy(mp, ccp->data, sizeof(struct mouse_pointer));
		ret = my_ibus_get_ret(dev);
	}

	return ret;
}
